@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalsimplewarp.cpp 11880 2007-08-13 15:02:33Z mloskot $
+ * $Id: gdalsimplewarp.cpp 15433 2008-09-24 15:11:36Z warmerdam $
  *
  * Project:  Mapinfo Image Warper
  * Purpose:  Simple (source in memory) warp algorithm.
@@ -31,7 +31,7 @@
 #include "gdal_alg.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: gdalsimplewarp.cpp 11880 2007-08-13 15:02:33Z mloskot $");
+CPL_CVSID("$Id: gdalsimplewarp.cpp 15433 2008-09-24 15:11:36Z warmerdam $");
 
 static void 
 GDALSimpleWarpRemapping( int nBandCount, GByte **papabySrcData, 
@@ -125,7 +125,7 @@ GDALSimpleImageWarp( GDALDatasetH hSrcDS, GDALDatasetH hDstDS,
     int   nSrcYSize = GDALGetRasterYSize(hSrcDS);
     GByte **papabySrcData;
 
-    papabySrcData = (GByte **) CPLCalloc(nBandCount,sizeof(int));
+    papabySrcData = (GByte **) CPLCalloc(nBandCount,sizeof(GByte*));
     for( iBand = 0; iBand < nBandCount; iBand++ )
     {
         papabySrcData[iBand] = (GByte *) VSIMalloc(nSrcXSize*nSrcYSize);
@@ -149,7 +149,7 @@ GDALSimpleImageWarp( GDALDatasetH hSrcDS, GDALDatasetH hDstDS,
     int nDstYSize = GDALGetRasterYSize( hDstDS );
     GByte **papabyDstLine;
 
-    papabyDstLine = (GByte **) CPLCalloc(nBandCount,sizeof(int));
+    papabyDstLine = (GByte **) CPLCalloc(nBandCount,sizeof(GByte*));
     
     for( iBand = 0; iBand < nBandCount; iBand++ )
         papabyDstLine[iBand] = (GByte *) CPLMalloc( nDstXSize );
