@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalwarper.cpp 10646 2007-01-18 02:38:10Z warmerdam $
+ * $Id: gdalwarper.cpp 13804 2008-02-17 05:30:09Z warmerdam $
  *
  * Project:  High Performance Image Reprojector
  * Purpose:  Implementation of high level convenience APIs for warper.
@@ -31,7 +31,7 @@
 #include "cpl_string.h"
 #include "cpl_minixml.h"
 
-CPL_CVSID("$Id: gdalwarper.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
+CPL_CVSID("$Id: gdalwarper.cpp 13804 2008-02-17 05:30:09Z warmerdam $");
 
 /************************************************************************/
 /*                         GDALReprojectImage()                         */
@@ -83,15 +83,6 @@ GDALReprojectImage( GDALDatasetH hSrcDS, const char *pszSrcWKT,
 {
     GDALWarpOptions *psWOptions;
 
-/* -------------------------------------------------------------------- */
-/*      Default a few parameters.                                       */
-/* -------------------------------------------------------------------- */
-    if( pszSrcWKT == NULL )
-        pszSrcWKT = GDALGetProjectionRef( hSrcDS );
-
-    if( pszDstWKT == NULL )
-        pszDstWKT = pszSrcWKT;
-
 /* -------------------------------------------------------------------- */
 /*      Setup a reprojection based transformer.                         */
 /* -------------------------------------------------------------------- */
@@ -240,6 +231,8 @@ CPLErr CPL_STDCALL GDALCreateAndReprojectImage(
     GDALWarpOptions *psOptions )
     
 {
+    VALIDATE_POINTER1( hSrcDS, "GDALCreateAndReprojectImage", CE_Failure );
+
 /* -------------------------------------------------------------------- */
 /*      Default a few parameters.                                       */
 /* -------------------------------------------------------------------- */
@@ -689,6 +682,8 @@ GDALWarpOptions * CPL_STDCALL GDALCreateWarpOptions()
 void CPL_STDCALL GDALDestroyWarpOptions( GDALWarpOptions *psOptions )
 
 {
+    VALIDATE_POINTER0( psOptions, "GDALDestroyWarpOptions" );
+
     CSLDestroy( psOptions->papszWarpOptions );
     CPLFree( psOptions->panSrcBands );
     CPLFree( psOptions->panDstBands );
@@ -759,7 +754,7 @@ GDALSerializeWarpOptions( const GDALWarpOptions *psWO )
 /* -------------------------------------------------------------------- */
     CPLCreateXMLElementAndValue( 
         psTree, "WarpMemoryLimit", 
-        CPLSPrintf("%g", psWO->dfWarpMemoryLimit ) );
+        CPLString().Printf("%g", psWO->dfWarpMemoryLimit ) );
 
 /* -------------------------------------------------------------------- */
 /*      ResampleAlg                                                     */
@@ -774,6 +769,8 @@ GDALSerializeWarpOptions( const GDALWarpOptions *psWO )
         pszAlgName = "Cubic";
     else if( psWO->eResampleAlg == GRA_CubicSpline )
         pszAlgName = "CubicSpline";
+    else if( psWO->eResampleAlg == GRA_Lanczos )
+        pszAlgName = "Lanczos";
     else
         pszAlgName = "Unknown";
 
@@ -862,31 +859,31 @@ GDALSerializeWarpOptions( const GDALWarpOptions *psWO )
         if( psWO->panSrcBands != NULL )
             CPLCreateXMLNode( 
                 CPLCreateXMLNode( psBand, CXT_Attribute, "src" ),
-                CXT_Text, CPLSPrintf( "%d", psWO->panSrcBands[i] ) );
+                CXT_Text, CPLString().Printf( "%d", psWO->panSrcBands[i] ) );
         if( psWO->panDstBands != NULL )
             CPLCreateXMLNode( 
                 CPLCreateXMLNode( psBand, CXT_Attribute, "dst" ),
-                CXT_Text, CPLSPrintf( "%d", psWO->panDstBands[i] ) );
+                CXT_Text, CPLString().Printf( "%d", psWO->panDstBands[i] ) );
         
         if( psWO->padfSrcNoDataReal != NULL )
             CPLCreateXMLElementAndValue( 
                 psBand, "SrcNoDataReal", 
-                CPLSPrintf( "%.16g", psWO->padfSrcNoDataReal[i] ) );
+                CPLString().Printf( "%.16g", psWO->padfSrcNoDataReal[i] ) );
 
         if( psWO->padfSrcNoDataImag != NULL )
             CPLCreateXMLElementAndValue( 
                 psBand, "SrcNoDataImag", 
-                CPLSPrintf( "%.16g", psWO->padfSrcNoDataImag[i] ) );
+                CPLString().Printf( "%.16g", psWO->padfSrcNoDataImag[i] ) );
 
         if( psWO->padfDstNoDataReal != NULL )
             CPLCreateXMLElementAndValue( 
                 psBand, "DstNoDataReal", 
-                CPLSPrintf( "%.16g", psWO->padfDstNoDataReal[i] ) );
+                CPLString().Printf( "%.16g", psWO->padfDstNoDataReal[i] ) );
 
         if( psWO->padfDstNoDataImag != NULL )
             CPLCreateXMLElementAndValue( 
                 psBand, "DstNoDataImag", 
-                CPLSPrintf( "%.16g", psWO->padfDstNoDataImag[i] ) );
+                CPLString().Printf( "%.16g", psWO->padfDstNoDataImag[i] ) );
     }
 
 /* -------------------------------------------------------------------- */
@@ -895,12 +892,12 @@ GDALSerializeWarpOptions( const GDALWarpOptions *psWO )
     if( psWO->nSrcAlphaBand > 0 )
         CPLCreateXMLElementAndValue( 
             psTree, "SrcAlphaBand", 
-            CPLSPrintf( "%d", psWO->nSrcAlphaBand ) );
+            CPLString().Printf( "%d", psWO->nSrcAlphaBand ) );
 
     if( psWO->nDstAlphaBand > 0 )
         CPLCreateXMLElementAndValue( 
             psTree, "DstAlphaBand", 
-            CPLSPrintf( "%d", psWO->nDstAlphaBand ) );
+            CPLString().Printf( "%d", psWO->nDstAlphaBand ) );
 
     return psTree;
 }
@@ -950,6 +947,8 @@ GDALWarpOptions * CPL_STDCALL GDALDeserializeWarpOptions( CPLXMLNode *psTree )
         psWO->eResampleAlg = GRA_Cubic;
     else if( EQUAL(pszValue,"CubicSpline") )
         psWO->eResampleAlg = GRA_CubicSpline;
+    else if( EQUAL(pszValue,"Lanczos") )
+        psWO->eResampleAlg = GRA_Lanczos;
     else if( EQUAL(pszValue,"Default") )
         /* leave as is */;
     else
