@@ -1,5 +1,5 @@
 /* ****************************************************************************
- * $Id: gdal_grid.cpp 13020 2007-11-25 16:33:58Z dron $
+ * $Id: gdal_grid.cpp 15054 2008-07-27 17:30:05Z rouault $
  *
  * Project:  GDAL Utilities
  * Purpose:  GDAL scattered data gridding (interpolation) tool
@@ -36,7 +36,7 @@
 #include "ogr_spatialref.h"
 #include "ogr_api.h"
 
-CPL_CVSID("$Id: gdal_grid.cpp 13020 2007-11-25 16:33:58Z dron $");
+CPL_CVSID("$Id: gdal_grid.cpp 15054 2008-07-27 17:30:05Z rouault $");
 
 static const char szAlgNameInvDist[] = "invdist";
 static const char szAlgNameAverage[] = "average";
@@ -258,9 +258,9 @@ static CPLErr ParseAlgorithmAndOptions( const char *pszAlgoritm,
 
 static void ProcessLayer( OGRLayerH hSrcLayer, GDALDatasetH hDstDS,
                           GUInt32 nXSize, GUInt32 nYSize, int nBand,
-                          int bIsXExtentSet, int bIsYExtentSet,
-                          double dfXMin, double dfXMax,
-                          double dfYMin, double dfYMax,
+                          int& bIsXExtentSet, int& bIsYExtentSet,
+                          double& dfXMin, double& dfXMax,
+                          double& dfYMin, double& dfYMax,
                           GDALDataType eType,
                           GDALGridAlgorithm eAlgorithm, void *pOptions,
                           int bQuiet, GDALProgressFunc pfnProgress )
@@ -282,8 +282,9 @@ static void ProcessLayer( OGRLayerH hSrcLayer, GDALDatasetH hDstDS,
         hGeom = OGR_F_GetGeometryRef( hFeat );
 
         // FIXME: handle collections
-        if ( OGR_G_GetGeometryType( hGeom ) == wkbPoint
-             || OGR_G_GetGeometryType( hGeom ) == wkbPoint25D )
+        if ( hGeom != NULL &&
+             (OGR_G_GetGeometryType( hGeom ) == wkbPoint
+              || OGR_G_GetGeometryType( hGeom ) == wkbPoint25D) )
         {
             adfX.push_back( OGR_G_GetX( hGeom, 0 ) );
             adfY.push_back( OGR_G_GetY( hGeom, 0 ) );
@@ -294,6 +295,13 @@ static void ProcessLayer( OGRLayerH hSrcLayer, GDALDatasetH hDstDS,
         OGR_F_Destroy( hFeat );
     }
 
+    if (adfX.size() == 0)
+    {
+        printf( "No point geometry found on layer %s, skipping.\n",
+                OGR_FD_GetName( OGR_L_GetLayerDefn( hSrcLayer ) ) );
+       return;
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Compute grid geometry.                                          */
 /* -------------------------------------------------------------------- */
@@ -302,12 +310,14 @@ static void ProcessLayer( OGRLayerH hSrcLayer, GDALDatasetH hDstDS,
     {
         dfXMin = *std::min_element(adfX.begin(), adfX.end());
         dfXMax = *std::max_element(adfX.begin(), adfX.end());
+        bIsXExtentSet = TRUE;
     }
 
     if ( !bIsYExtentSet )
     {
         dfYMin = *std::min_element(adfY.begin(), adfY.end());
         dfYMax = *std::max_element(adfY.begin(), adfY.end());
+        bIsYExtentSet = TRUE;
     }
 
 /* -------------------------------------------------------------------- */
@@ -352,18 +362,26 @@ static void ProcessLayer( OGRLayerH hSrcLayer, GDALDatasetH hDstDS,
                                           (double)++nBlock / nBlockCount,
                                           pfnProgress, NULL );
 
+            int nXRequest = nBlockXSize;
+            if (nXOffset + nXRequest > nXSize)
+                nXRequest = nXSize - nXOffset;
+
+            int nYRequest = nBlockYSize;
+            if (nYOffset + nYRequest > nYSize)
+                nYRequest = nYSize - nYOffset;
+
             GDALGridCreate( eAlgorithm, pOptions,
                             adfX.size(), &(adfX[0]), &(adfY[0]), &(adfZ[0]),
                             dfXMin + dfDeltaX * nXOffset,
-                            dfXMin + dfDeltaX * (nXOffset + nBlockXSize),
+                            dfXMin + dfDeltaX * (nXOffset + nXRequest),
                             dfYMin + dfDeltaY * nYOffset,
-                            dfYMin + dfDeltaY * (nYOffset + nBlockYSize),
-                            nBlockXSize, nBlockYSize, eType, pData,
+                            dfYMin + dfDeltaY * (nYOffset + nYRequest),
+                            nXRequest, nYRequest, eType, pData,
                             GDALScaledProgress, pScaledProgress );
 
             GDALRasterIO( hBand, GF_Write, nXOffset, nYOffset,
-                          nBlockXSize, nBlockYSize, pData,
-                          nBlockXSize, nBlockYSize, eType, 0, 0 );
+                          nXRequest, nYRequest, pData,
+                          nXRequest, nYRequest, eType, 0, 0 );
 
             GDALDestroyScaledProgress( pScaledProgress );
         }
@@ -387,7 +405,7 @@ int main( int argc, char ** argv )
     GUInt32         nXSize = 0, nYSize = 0;
     double          dfXMin = 0.0, dfXMax = 0.0, dfYMin = 0.0, dfYMax = 0.0;
     int             bIsXExtentSet = FALSE, bIsYExtentSet = FALSE;
-    GDALGridAlgorithm eAlgorithm;
+    GDALGridAlgorithm eAlgorithm = GGA_InverseDistanceToAPower;
     void            *pOptions = NULL;
     char            *pszOutputSRS = NULL;
     int             bQuiet = FALSE;
