@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: aaigriddataset.cpp 12461 2007-10-17 16:56:16Z mloskot $
+ * $Id: aaigriddataset.cpp 14477 2008-05-16 20:17:25Z rouault $
  *
  * Project:  GDAL
  * Purpose:  Implements Arc/Info ASCII Grid Format.
@@ -29,10 +29,11 @@
 
 #include "gdal_pam.h"
 #include <ctype.h>
+#include <limits.h>
 #include "cpl_string.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: aaigriddataset.cpp 12461 2007-10-17 16:56:16Z mloskot $");
+CPL_CVSID("$Id: aaigriddataset.cpp 14477 2008-05-16 20:17:25Z rouault $");
 
 CPL_C_START
 void    GDALRegister_AAIGrid(void);
@@ -57,6 +58,7 @@ class CPL_DLL AAIGDataset : public GDALPamDataset
 
     double      adfGeoTransform[6];
     char        **papszPrj;
+    CPLString   osPrjFilename;
     char        *pszProjection;
 
     int         bNoDataSet;
@@ -74,6 +76,8 @@ class CPL_DLL AAIGDataset : public GDALPamDataset
                 AAIGDataset();
                 ~AAIGDataset();
 
+    virtual char **GetFileList(void);
+
     static GDALDataset *Open( GDALOpenInfo * );
     static CPLErr       Delete( const char *pszFilename );
     static CPLErr       Remove( const char *pszFilename, int bRepError );
@@ -121,7 +125,14 @@ AAIGRasterBand::AAIGRasterBand( AAIGDataset *poDS, int nDataStart,
     nBlockYSize = 1;
 
     panLineOffset = 
-        (GUIntBig *) CPLCalloc( poDS->nRasterYSize, sizeof(GUIntBig) );
+        (GUIntBig *) VSICalloc( poDS->nRasterYSize, sizeof(GUIntBig) );
+    if (panLineOffset == NULL)
+    {
+        CPLError(CE_Failure, CPLE_OutOfMemory,
+                 "AAIGRasterBand::AAIGRasterBand : Out of memory (nRasterYSize = %d)",
+                 poDS->nRasterYSize);
+        return;
+    }
     panLineOffset[0] = nDataStart;
 }
 
@@ -147,7 +158,7 @@ CPLErr AAIGRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
     int         iPixel;
 
     if( nBlockYOff < 0 || nBlockYOff > poODS->nRasterYSize - 1 
-        || nBlockXOff != 0 )
+        || nBlockXOff != 0 || panLineOffset == NULL)
         return CE_Failure;
 
     if( panLineOffset[nBlockYOff] == 0 )
@@ -343,6 +354,21 @@ char AAIGDataset::Getc()
     return achReadBuf[nOffsetInBuffer++];
 }
 
+/************************************************************************/
+/*                            GetFileList()                             */
+/************************************************************************/
+
+char **AAIGDataset::GetFileList()
+
+{
+    char **papszFileList = GDALPamDataset::GetFileList();
+
+    if( papszPrj != NULL )
+        papszFileList = CSLAddString( papszFileList, osPrjFilename );
+
+    return papszFileList;
+}
+
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
@@ -350,9 +376,10 @@ char AAIGDataset::Getc()
 GDALDataset *AAIGDataset::Open( GDALOpenInfo * poOpenInfo )
 
 {
-    int         i, j;
-    GDALDataType eDataType;
-    char        **papszTokens;
+    int i = 0;
+    int j = 0;
+    char **papszTokens = NULL;
+    GDALDataType eDataType = GDT_Int16;
 
 /* -------------------------------------------------------------------- */
 /*      Does this look like an AI grid file?                            */
@@ -383,7 +410,8 @@ GDALDataset *AAIGDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Parse the header.                                               */
 /* -------------------------------------------------------------------- */
-    double dfCellDX, dfCellDY;
+    double dfCellDX = 0;
+    double dfCellDY = 0;
 
     if ( (i = CSLFindString( papszTokens, "ncols" )) < 0 )
     {
@@ -450,8 +478,10 @@ GDALDataset *AAIGDataset::Open( GDALOpenInfo * poOpenInfo )
 
     if( (i = CSLFindString( papszTokens, "NODATA_value" )) >= 0 )
     {
+        const char* pszNoData = papszTokens[i + 1];
+
         poDS->bNoDataSet = TRUE;
-        poDS->dfNoDataValue = atof(papszTokens[i + 1]);
+        poDS->dfNoDataValue = atof(pszNoData);
     }
     
     CSLDestroy( papszTokens );
@@ -499,58 +529,71 @@ GDALDataset *AAIGDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Recognize the type of data.										*/
 /* -------------------------------------------------------------------- */
-
     CPLAssert( NULL != poDS->fp );
 
-    /* Default value type. */
-    eDataType = GDT_Int16; 
-
-    /* Allocate 100K chunk + 1 extra byte for NULL character. */
-    const size_t nChunkSize = 1024 * 100;
-    GByte* pabyChunk = (GByte *) CPLCalloc( nChunkSize + 1, sizeof(GByte) );
-    pabyChunk[nChunkSize] = '\0';
-
-    VSIFSeekL( poDS->fp, nStartOfData, SEEK_SET );
-	
-    /* Scan for dot in subsequent chunks of data. */
-    while( !VSIFEofL( poDS->fp) )
+    /* Use bigger data type. */
+    if( poDS->bNoDataSet
+        && ( SHRT_MIN > poDS->dfNoDataValue || poDS->dfNoDataValue > SHRT_MAX) )
     {
-        VSIFReadL( pabyChunk, sizeof(GByte), nChunkSize, poDS->fp );
-        CPLAssert( pabyChunk[nChunkSize] == '\0' );
-        
-        if( strchr( (const char *)pabyChunk, '.' ) != NULL )
+        eDataType = GDT_Float32; 
+    }
+    else
+    {
+        /* Default value type. */
+        eDataType = GDT_Int16;
+
+        /* Allocate 100K chunk + 1 extra byte for NULL character. */
+        const size_t nChunkSize = 1024 * 100;
+        GByte* pabyChunk = (GByte *) CPLCalloc( nChunkSize + 1, sizeof(GByte) );
+        pabyChunk[nChunkSize] = '\0';
+
+        VSIFSeekL( poDS->fp, nStartOfData, SEEK_SET );
+
+        /* Scan for dot in subsequent chunks of data. */
+        while( !VSIFEofL( poDS->fp) )
         {
-            eDataType = GDT_Float32;
-            break;
+            VSIFReadL( pabyChunk, sizeof(GByte), nChunkSize, poDS->fp );
+            CPLAssert( pabyChunk[nChunkSize] == '\0' );
+
+            if( strchr( (const char *)pabyChunk, '.' ) != NULL )
+            {
+                eDataType = GDT_Float32;
+                break;
+            }
         }
+
+        /* Deallocate chunk. */
+        VSIFree( pabyChunk );
     }
-    
-    /* Deallocate chunk. */
-    VSIFree( pabyChunk );
 
 /* -------------------------------------------------------------------- */
 /*      Create band information objects.                                */
 /* -------------------------------------------------------------------- */
-    poDS->SetBand( 1, new AAIGRasterBand( poDS, nStartOfData, eDataType ) );
+    AAIGRasterBand* band = new AAIGRasterBand( poDS, nStartOfData, eDataType );
+    poDS->SetBand( 1, band );
+    if (band->panLineOffset == NULL)
+    {
+        delete poDS;
+        return NULL;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Try to read projection file.                                    */
 /* -------------------------------------------------------------------- */
     char        *pszDirname, *pszBasename;
-    const char  *pszPrjFilename;
     VSIStatBufL   sStatBuf;
 
     pszDirname = CPLStrdup(CPLGetPath(poOpenInfo->pszFilename));
     pszBasename = CPLStrdup(CPLGetBasename(poOpenInfo->pszFilename));
 
-    pszPrjFilename = CPLFormFilename( pszDirname, pszBasename, "prj" );
-    int nRet = VSIStatL( pszPrjFilename, &sStatBuf );
+    poDS->osPrjFilename = CPLFormFilename( pszDirname, pszBasename, "prj" );
+    int nRet = VSIStatL( poDS->osPrjFilename, &sStatBuf );
 
 #ifndef WIN32
     if( nRet != 0 )
     {
-        pszPrjFilename = CPLFormFilename( pszDirname, pszBasename, "PRJ" );
-        nRet = VSIStatL( pszPrjFilename, &sStatBuf );
+        poDS->osPrjFilename = CPLFormFilename( pszDirname, pszBasename, "PRJ" );
+        nRet = VSIStatL( poDS->osPrjFilename, &sStatBuf );
     }
 #endif
 
@@ -558,9 +601,10 @@ GDALDataset *AAIGDataset::Open( GDALOpenInfo * poOpenInfo )
     {
         OGRSpatialReference     oSRS;
 
-        poDS->papszPrj = CSLLoad( pszPrjFilename );
+        poDS->papszPrj = CSLLoad( poDS->osPrjFilename );
 
-        CPLDebug( "AAIGrid", "Loaded SRS from %s", pszPrjFilename );
+        CPLDebug( "AAIGrid", "Loaded SRS from %s", 
+                  poDS->osPrjFilename.c_str() );
 
         if( oSRS.importFromESRI( poDS->papszPrj ) == OGRERR_NONE )
         {
@@ -679,7 +723,7 @@ AAIGCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
                  "cellsize     %.12f\n", 
                  nXSize, nYSize, 
                  adfGeoTransform[0], 
-                 adfGeoTransform[3]- nYSize * adfGeoTransform[1],
+                 adfGeoTransform[3] - nYSize * adfGeoTransform[1],
                  adfGeoTransform[1] );
     else
     {
@@ -699,7 +743,7 @@ AAIGCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
                  "dy           %.12f\n", 
                  nXSize, nYSize, 
                  adfGeoTransform[0], 
-                 adfGeoTransform[3]- nYSize * adfGeoTransform[5],
+                 adfGeoTransform[3] + nYSize * adfGeoTransform[5],
                  adfGeoTransform[1],
                  fabs(adfGeoTransform[5]) );
     }
@@ -722,27 +766,37 @@ AAIGCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 /* -------------------------------------------------------------------- */
 /*      Loop over image, copying image data.                            */
 /* -------------------------------------------------------------------- */
-    double      *padfScanline;
+    int         *panScanline = NULL;
+    double      *padfScanline = NULL;
+    int         bReadAsInt;
     int         iLine, iPixel;
     CPLErr      eErr = CE_None;
     
+    bReadAsInt = ( poBand->GetRasterDataType() == GDT_Byte 
+                || poBand->GetRasterDataType() == GDT_Int16
+                || poBand->GetRasterDataType() == GDT_UInt16
+                || poBand->GetRasterDataType() == GDT_Int32 );
+
     // Write scanlines to output file
-    padfScanline = (double *) CPLMalloc( nXSize *
-                                GDALGetDataTypeSize(GDT_CFloat64) / 8 );
+    if (bReadAsInt)
+        panScanline = (int *) CPLMalloc( nXSize *
+                                GDALGetDataTypeSize(GDT_Int32) / 8 );
+    else
+        padfScanline = (double *) CPLMalloc( nXSize *
+                                    GDALGetDataTypeSize(GDT_Float64) / 8 );
+
     for( iLine = 0; eErr == CE_None && iLine < nYSize; iLine++ )
     {
         eErr = poBand->RasterIO( GF_Read, 0, iLine, nXSize, 1, 
-                              padfScanline, nXSize, 1, GDT_CFloat64,
-                              sizeof(double), sizeof(double) * nXSize );
+                                 (bReadAsInt) ? (void*)panScanline : (void*)padfScanline,
+                                 nXSize, 1, (bReadAsInt) ? GDT_Int32 : GDT_Float64,
+                                 0, 0 );
 
-        if( poBand->GetRasterDataType() == GDT_Byte 
-            || poBand->GetRasterDataType() == GDT_Int16
-            || poBand->GetRasterDataType() == GDT_UInt16
-            || poBand->GetRasterDataType() == GDT_Int32 )
+        if( bReadAsInt )
         {
             for ( iPixel = 0; iPixel < nXSize; iPixel++ )
             {
-                sprintf( szHeader, " %d", (int) padfScanline[iPixel] );
+                sprintf( szHeader, " %d", panScanline[iPixel] );
                 if( VSIFWriteL( szHeader, strlen(szHeader), 1, fpImage ) != 1 )
                 {
                     eErr = CE_Failure;
@@ -777,6 +831,7 @@ AAIGCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
         }
     }
 
+    CPLFree( panScanline );
     CPLFree( padfScanline );
     VSIFCloseL( fpImage );
 
@@ -789,7 +844,7 @@ AAIGCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
     if( !EQUAL( pszOriginalProjection, "" ) )
     {
         char                    *pszDirname, *pszBasename;
-        const char              *pszPrjFilename;
+        char                    *pszPrjFilename;
         char                    *pszESRIProjection = NULL;
         FILE                    *fp;
         OGRSpatialReference     oSRS;
@@ -797,18 +852,26 @@ AAIGCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
         pszDirname = CPLStrdup( CPLGetPath(pszFilename) );
         pszBasename = CPLStrdup( CPLGetBasename(pszFilename) );
 
-        pszPrjFilename = CPLFormFilename( pszDirname, pszBasename, "prj" );
+        pszPrjFilename = CPLStrdup( CPLFormFilename( pszDirname, pszBasename, "prj" ) );
         fp = VSIFOpenL( pszPrjFilename, "wt" );
-        
-        oSRS.importFromWkt( (char **) &pszOriginalProjection );
-        oSRS.morphToESRI();
-        oSRS.exportToWkt( &pszESRIProjection );
-        VSIFWriteL( pszESRIProjection, 1, strlen(pszESRIProjection), fp );
+        if (fp != NULL)
+        {
+            oSRS.importFromWkt( (char **) &pszOriginalProjection );
+            oSRS.morphToESRI();
+            oSRS.exportToWkt( &pszESRIProjection );
+            VSIFWriteL( pszESRIProjection, 1, strlen(pszESRIProjection), fp );
 
-        VSIFCloseL( fp );
+            VSIFCloseL( fp );
+            CPLFree( pszESRIProjection );
+        }
+        else
+        {
+            CPLError( CE_Failure, CPLE_FileIO, 
+                      "Unable to create file %s.\n", pszPrjFilename );
+        }
         CPLFree( pszDirname );
         CPLFree( pszBasename );
-        CPLFree( pszESRIProjection );
+        CPLFree( pszPrjFilename );
     }
     
 /* -------------------------------------------------------------------- */
@@ -823,49 +886,6 @@ AAIGCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
     return poDS;
 }
 
-/************************************************************************/
-/*                               Remove()                               */
-/*    Called from the Delete()                                          */
-/************************************************************************/
-
-CPLErr AAIGDataset::Remove( const char * pszFilename, int bRepError )
-
-{
-    VSIStatBufL      sStat;
-
-    if( VSIStatL( pszFilename, &sStat ) == 0 && VSI_ISREG( sStat.st_mode ) )
-    {
-        if( VSIUnlink( pszFilename ) == 0 )
-            return CE_None;
-        else
-        {
-            CPLError( CE_Failure, CPLE_AppDefined,
-                      "Attempt to unlink %s failed.\n", pszFilename );
-            return CE_Failure;
-        }
-    }
-    else if( bRepError )
-    {
-        
-        CPLError( CE_Failure, CPLE_AppDefined,
-                  "Unable to delete %s, not a file.\n", pszFilename );
-        return CE_Failure;
-    }
-    
-    return CE_None;
-}
-
-/************************************************************************/
-/*                               Delete()                               */
-/************************************************************************/
-
-CPLErr AAIGDataset::Delete( const char *pszFilename )
-
-{
-    Remove( CPLResetExtension( pszFilename, "prj" ), FALSE );
-    return Remove( pszFilename, TRUE );
-}
-
 /************************************************************************/
 /*                              OSR_GDS()                               */
 /************************************************************************/
@@ -932,7 +952,6 @@ void GDALRegister_AAIGrid()
 
         poDriver->pfnOpen = AAIGDataset::Open;
         poDriver->pfnCreateCopy = AAIGCreateCopy;
-        poDriver->pfnDelete = AAIGDataset::Delete;
         
         GetGDALDriverManager()->RegisterDriver( poDriver );
     }
