@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: grass57dataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $
+ * $Id: grass57dataset.cpp 12396 2007-10-13 10:02:17Z rouault $
  *
  * Project:  GRASS Driver
  * Purpose:  Implement GRASS raster read/write support
@@ -41,6 +41,7 @@ extern "C" {
 #undef class
 #endif
     
+#include <grass/version.h>
 #include <grass/gprojects.h>
 #include <grass/gis.h>
 }
@@ -51,7 +52,7 @@ extern "C" {
 
 #define GRASS_MAX_COLORS 100000  // what is the right value
 
-CPL_CVSID("$Id: grass57dataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
+CPL_CVSID("$Id: grass57dataset.cpp 12396 2007-10-13 10:02:17Z rouault $");
 
 CPL_C_START
 void	GDALRegister_GRASS(void);
@@ -785,7 +786,11 @@ bool GRASSDataset::SplitPath( char *path, char **gisdbase, char **location,
 /*                                Open()                                */
 /************************************************************************/
 
+#if GRASS_VERSION_MAJOR  >= 6 && GRASS_VERSION_MINOR  >= 3
+typedef int (*GrassErrorHandler)(const char *, int);
+#else
 typedef int (*GrassErrorHandler)(char *, int);
+#endif
 
 GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
 
@@ -964,6 +969,9 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
 void GDALRegister_GRASS()
 {
     GDALDriver	*poDriver;
+    
+    if (! GDAL_CHECK_VERSION("GDAL/GRASS57 driver"))
+        return;
 
     if( GDALGetDriverByName( "GRASS" ) == NULL )
     {
