@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: hfafield.cpp 13725 2008-02-07 20:08:29Z warmerdam $
+ * $Id: hfafield.cpp 14762 2008-06-24 21:39:41Z warmerdam $
  *
  * Project:  Erdas Imagine (.img) Translator
  * Purpose:  Implementation of the HFAField class for managing information
@@ -30,7 +30,7 @@
 
 #include "hfa_p.h"
 
-CPL_CVSID("$Id: hfafield.cpp 13725 2008-02-07 20:08:29Z warmerdam $");
+CPL_CVSID("$Id: hfafield.cpp 14762 2008-06-24 21:39:41Z warmerdam $");
 
 #define MAX_ENTRY_REPORT   16
                            
@@ -407,9 +407,15 @@ HFAField::SetInstValue( const char * pszField, int nIndexValue,
             return CE_Failure;
         }
 
-        nOffset = nCount;
+        // we will update the object count iff we are writing beyond the end
+        memcpy( &nOffset, pabyData, 4 );
         HFAStandard( 4, &nOffset );
-        memcpy( pabyData, &nOffset, 4 );
+        if( nOffset < nCount )
+        {
+            nOffset = nCount;
+            HFAStandard( 4, &nOffset );
+            memcpy( pabyData, &nOffset, 4 );
+        }
 
         if( pValue == NULL )
             nOffset = 0;
