@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: rawdataset.cpp 13200 2007-12-02 20:18:12Z rouault $
+ * $Id: rawdataset.cpp 15395 2008-09-19 18:21:00Z rouault $
  *
  * Project:  Generic Raw Binary Driver
  * Purpose:  Implementation of RawDataset and RawRasterBand classes.
@@ -31,7 +31,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: rawdataset.cpp 13200 2007-12-02 20:18:12Z rouault $");
+CPL_CVSID("$Id: rawdataset.cpp 15395 2008-09-19 18:21:00Z rouault $");
 
 /************************************************************************/
 /*                           RawRasterBand()                            */
@@ -75,7 +75,13 @@ RawRasterBand::RawRasterBand( GDALDataset *poDS, int nBand,
 /* -------------------------------------------------------------------- */
     nLoadedScanline = -1;
     nLineSize = nPixelOffset * nBlockXSize;
-    pLineBuffer = CPLMalloc( nLineSize );
+    pLineBuffer = (nLineSize <= 0 ) ? NULL : VSIMalloc( nLineSize );
+    if (pLineBuffer == NULL)
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "Could not allocate line buffer : nPixelOffset=%d, nBlockXSize=%d",
+                 nPixelOffset, nBlockXSize);
+    }
 }
 
 /************************************************************************/
@@ -120,7 +126,13 @@ RawRasterBand::RawRasterBand( FILE * fpRaw, vsi_l_offset nImgOffset,
 /* -------------------------------------------------------------------- */
     nLoadedScanline = -1;
     nLineSize = nPixelOffset * nBlockXSize;
-    pLineBuffer = CPLMalloc( nLineSize );
+    pLineBuffer = (nLineSize <= 0 ) ? NULL : VSIMalloc( nLineSize );
+    if (pLineBuffer == NULL)
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "Could not allocate line buffer : nPixelOffset=%d, nBlockXSize=%d",
+                 nPixelOffset, nBlockXSize);
+    }
 }
 
 /************************************************************************/
@@ -197,6 +209,9 @@ CPLErr RawRasterBand::FlushCache()
 CPLErr RawRasterBand::AccessLine( int iLine )
 
 {
+    if (pLineBuffer == NULL)
+        return CE_Failure;
+
     if( nLoadedScanline == iLine )
         return CE_None;
 
@@ -273,7 +288,10 @@ CPLErr RawRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 
     CPLAssert( nBlockXOff == 0 );
 
-    AccessLine( nBlockYOff );
+    if (pLineBuffer == NULL)
+        return CE_Failure;
+
+    eErr = AccessLine( nBlockYOff );
     
 /* -------------------------------------------------------------------- */
 /*      Copy data from disk buffer to user block buffer.                */
@@ -297,6 +315,9 @@ CPLErr RawRasterBand::IWriteBlock( int nBlockXOff, int nBlockYOff,
 
     CPLAssert( nBlockXOff == 0 );
 
+    if (pLineBuffer == NULL)
+        return CE_Failure;
+
 /* -------------------------------------------------------------------- */
 /*      If the data for this band is completely contiguous we don't     */
 /*      have to worry about pre-reading from disk.                      */
