@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: usgsdem_create.cpp 10646 2007-01-18 02:38:10Z warmerdam $
+ * $Id: usgsdem_create.cpp 12490 2007-10-22 13:53:32Z dron $
  *
  * Project:  USGS DEM Driver
  * Purpose:  CreateCopy() implementation.
@@ -36,7 +36,7 @@
 #include "gdalwarper.h"
 #include "cpl_csv.h"
 
-CPL_CVSID("$Id: usgsdem_create.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
+CPL_CVSID("$Id: usgsdem_create.cpp 12490 2007-10-22 13:53:32Z dron $");
 
 typedef struct 
 {
@@ -845,7 +845,7 @@ USGSDEM_LookupNTSByLoc( double dfULLong, double dfULLat,
     fpNTS = VSIFOpen( pszNTSFilename, "rb" );
     if( fpNTS == NULL )
     {
-        CPLDebug( "Unable to find NTS mapsheet lookup file: %s", 
+        CPLError( CE_Failure, CPLE_FileIO, "Unable to find NTS mapsheet lookup file: %s", 
                   pszNTSFilename );
         return FALSE;
     }
@@ -902,7 +902,7 @@ USGSDEM_LookupNTSByTile( const char *pszTile, char *pszName,
     fpNTS = VSIFOpen( pszNTSFilename, "rb" );
     if( fpNTS == NULL )
     {
-        CPLDebug( "Unable to find NTS mapsheet lookup file: %s", 
+        CPLError( CE_Failure, CPLE_FileIO, "Unable to find NTS mapsheet lookup file: %s", 
                   pszNTSFilename );
         return FALSE;
     }
@@ -971,7 +971,7 @@ static int USGSDEMProductSetup_CDED50K( USGSDEMWriteInfo *psWInfo )
             return FALSE;
 
         if( EQUALN(pszNTS+6,"e",1) )
-            dfULX += 0.25;
+            dfULX += (( dfULY < 68.1 ) ? 0.25 : ( dfULY < 80.1 ) ? 0.5 : 1);
     }
 
     // Try looking up TOPLEFT as a NTS mapsheet name.
@@ -987,7 +987,7 @@ static int USGSDEMProductSetup_CDED50K( USGSDEMWriteInfo *psWInfo )
             return FALSE;
 
         if( EQUAL(pszTOPLEFT+6,"e") )
-            dfULX += 0.25;
+            dfULX += (( dfULY < 68.1 ) ? 0.25 : ( dfULY < 80.1 ) ? 0.5 : 1);
     }
 
     // Assume TOPLEFT is a long/lat corner.
@@ -1028,7 +1028,7 @@ static int USGSDEMProductSetup_CDED50K( USGSDEMWriteInfo *psWInfo )
             return FALSE;
 
         if( EQUALN(psWInfo->pszFilename+7,"e",1) )
-            dfULX += 0.25;
+            dfULX += (( dfULY < 68.1 ) ? 0.25 : ( dfULY < 80.1 ) ? 0.5 : 1);
     }
              
     else if( strlen(psWInfo->pszFilename) == 14 
@@ -1044,7 +1044,7 @@ static int USGSDEMProductSetup_CDED50K( USGSDEMWriteInfo *psWInfo )
             return FALSE;
 
         if( EQUALN(psWInfo->pszFilename+9,"e",1) )
-            dfULX += 0.25;
+            dfULX += (( dfULY < 68.1 ) ? 0.25 : ( dfULY < 80.1 ) ? 0.5 : 1);
     }
 
 /* -------------------------------------------------------------------- */
@@ -1450,8 +1450,11 @@ USGSDEMCreateCopy( const char *pszFilename, GDALDataset *poSrcDS,
      }
      else 
      {
-         sWInfo.dfElevStepSize = CPLScanDouble(
-                 zResolution, strlen(zResolution), NULL);
+         // XXX: We are using atof() here instead of CPLAtof() because
+         // zResolution value comes from user's input and supposed to be
+         // written according to user's current locale. atof() honors locale
+         // setting, CPLAtof() is not.
+         sWInfo.dfElevStepSize = atof( zResolution );
          if ( sWInfo.dfElevStepSize <= 0 )
          {
              /* don't allow negative values */
@@ -1468,12 +1471,18 @@ USGSDEMCreateCopy( const char *pszFilename, GDALDataset *poSrcDS,
     if( pszProduct == NULL || EQUAL(pszProduct,"DEFAULT") )
     {
         if ( !USGSDEMProductSetup_DEFAULT( &sWInfo ) )
+        {
+            USGSDEMWriteCleanup( &sWInfo );
             return NULL;
+        }
     }
     else if( EQUAL(pszProduct,"CDED50K") )
     {
         if( !USGSDEMProductSetup_CDED50K( &sWInfo ) )
+        {
+            USGSDEMWriteCleanup( &sWInfo );
             return NULL;
+        }
     }
     else
     {
