@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: vrtsources.cpp 11621 2007-06-08 04:05:14Z warmerdam $
+ * $Id: vrtsources.cpp 11864 2007-08-09 11:43:07Z warmerdam $
  *
  * Project:  Virtual GDAL Datasets
  * Purpose:  Implementation of VRTSimpleSource, VRTFuncSource and 
@@ -32,7 +32,7 @@
 #include "cpl_minixml.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: vrtsources.cpp 11621 2007-06-08 04:05:14Z warmerdam $");
+CPL_CVSID("$Id: vrtsources.cpp 11864 2007-08-09 11:43:07Z warmerdam $");
 
 /************************************************************************/
 /* ==================================================================== */
@@ -303,12 +303,15 @@ VRTSimpleSource::GetSrcDstWindow( int nXOff, int nYOff, int nXSize, int nYSize,
                                   int *pnOutXSize, int *pnOutYSize )
 
 {
-    int bSrcWinSet = nSrcXOff != -1 || nSrcXSize != -1 
-        || nSrcYOff != -1 || nSrcYSize != -1;
     int bDstWinSet = nDstXOff != -1 || nDstXSize != -1 
         || nDstYOff != -1 || nDstYSize != -1;
 
+#ifdef DEBUG
+    int bSrcWinSet = nSrcXOff != -1 || nSrcXSize != -1 
+        || nSrcYOff != -1 || nSrcYSize != -1;
+
     CPLAssert( bSrcWinSet == bDstWinSet );
+#endif
 
 /* -------------------------------------------------------------------- */
 /*      If the input window completely misses the portion of the        */
