@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdaldrivermanager.cpp 10646 2007-01-18 02:38:10Z warmerdam $
+ * $Id: gdaldrivermanager.cpp 14081 2008-03-25 22:13:05Z tamas $
  *
  * Project:  GDAL Core
  * Purpose:  Implementation of GDALDriverManager class.
@@ -33,7 +33,7 @@
 #include "ogr_srs_api.h"
 #include "cpl_multiproc.h"
 
-CPL_CVSID("$Id: gdaldrivermanager.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
+CPL_CVSID("$Id: gdaldrivermanager.cpp 14081 2008-03-25 22:13:05Z tamas $");
 
 static const char *pszUpdatableINST_DATA = 
 "__INST_DATA_TARGET:                                                                                                                                      ";
@@ -75,7 +75,9 @@ GDALDriverManager * GetGDALDriverManager()
             poDM = new GDALDriverManager();
     }
 
-    return( (GDALDriverManager *) poDM );
+    CPLAssert( NULL != poDM );
+
+    return const_cast<GDALDriverManager *>( poDM );
 }
 
 /************************************************************************/
@@ -328,6 +330,8 @@ int GDALDriverManager::RegisterDriver( GDALDriver * poDriver )
 int CPL_STDCALL GDALRegisterDriver( GDALDriverH hDriver )
 
 {
+    VALIDATE_POINTER1( hDriver, "GDALRegisterDriver", 0 );
+
     return GetGDALDriverManager()->RegisterDriver( (GDALDriver *) hDriver );
 }
 
@@ -380,6 +384,8 @@ void GDALDriverManager::DeregisterDriver( GDALDriver * poDriver )
 void CPL_STDCALL GDALDeregisterDriver( GDALDriverH hDriver )
 
 {
+    VALIDATE_POINTER0( hDriver, "GDALDeregisterDriver" );
+
     GetGDALDriverManager()->DeregisterDriver( (GDALDriver *) hDriver );
 }
 
@@ -425,6 +431,8 @@ GDALDriver * GDALDriverManager::GetDriverByName( const char * pszName )
 GDALDriverH CPL_STDCALL GDALGetDriverByName( const char * pszName )
 
 {
+    VALIDATE_POINTER1( pszName, "GDALGetDriverByName", NULL );
+
     return( GetGDALDriverManager()->GetDriverByName( pszName ) );
 }
 
@@ -551,7 +559,7 @@ void GDALDriverManager::AutoLoadDrivers()
         {
             char szPluginDir[sizeof(szExecPath)+50];
             strcpy( szPluginDir, CPLGetDirname( szExecPath ) );
-            strcat( szPluginDir, "\\gdalplugins\\" );
+            strcat( szPluginDir, "\\gdalplugins" );
             papszSearchPath = CSLAddString( papszSearchPath, szPluginDir );
         }
         else
@@ -655,3 +663,5 @@ void CPL_STDCALL GDALDestroyDriverManager( void )
     if( poDM != NULL )
         delete poDM;
 }
+
+
