@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrfeature.cpp 11087 2007-03-27 03:55:12Z warmerdam $
+ * $Id: ogrfeature.cpp 14270 2008-04-12 13:28:42Z rouault $
  * 
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  The OGRFeature class implementation. 
@@ -31,7 +31,7 @@
 #include "ogr_api.h"
 #include "ogr_p.h"
 
-CPL_CVSID("$Id: ogrfeature.cpp 11087 2007-03-27 03:55:12Z warmerdam $");
+CPL_CVSID("$Id: ogrfeature.cpp 14270 2008-04-12 13:28:42Z rouault $");
 
 /************************************************************************/
 /*                             OGRFeature()                             */
@@ -99,6 +99,8 @@ OGRFeature::OGRFeature( OGRFeatureDefn * poDefnIn )
 OGRFeatureH OGR_F_Create( OGRFeatureDefnH hDefn )
 
 {
+    VALIDATE_POINTER1( hDefn, "OGR_F_Create", NULL );
+
     return (OGRFeatureH) new OGRFeature( (OGRFeatureDefn *) hDefn );
 }
 
@@ -255,6 +257,8 @@ void OGRFeature::DestroyFeature( OGRFeature *poFeature )
 OGRFeatureDefnH OGR_F_GetDefnRef( OGRFeatureH hFeat )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_GetDefnRef", NULL );
+
     return ((OGRFeature *) hFeat)->GetDefnRef();
 }
 
@@ -316,6 +320,8 @@ OGRErr OGRFeature::SetGeometryDirectly( OGRGeometry * poGeomIn )
 OGRErr OGR_F_SetGeometryDirectly( OGRFeatureH hFeat, OGRGeometryH hGeom )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_SetGeometryDirectly", CE_Failure );
+
     return ((OGRFeature *) hFeat)->SetGeometryDirectly((OGRGeometry *) hGeom);
 }
 
@@ -380,6 +386,8 @@ OGRErr OGRFeature::SetGeometry( OGRGeometry * poGeomIn )
 OGRErr OGR_F_SetGeometry( OGRFeatureH hFeat, OGRGeometryH hGeom )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_SetGeometry", CE_Failure );
+
     return ((OGRFeature *) hFeat)->SetGeometry((OGRGeometry *) hGeom);
 }
 
@@ -440,6 +448,8 @@ OGRGeometry *OGRFeature::StealGeometry()
 OGRGeometryH OGR_F_GetGeometryRef( OGRFeatureH hFeat )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_GetGeometryRef", NULL );
+
     return ((OGRFeature *) hFeat)->GetGeometryRef();
 }
 
@@ -497,6 +507,8 @@ OGRFeature *OGRFeature::Clone()
 OGRFeatureH OGR_F_Clone( OGRFeatureH hFeat )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_Clone", NULL );
+
     return (OGRFeatureH) ((OGRFeature *) hFeat)->Clone();
 }
 
@@ -532,6 +544,8 @@ OGRFeatureH OGR_F_Clone( OGRFeatureH hFeat )
 int OGR_F_GetFieldCount( OGRFeatureH hFeat )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_GetFieldCount", 0 );
+
     return ((OGRFeature *) hFeat)->GetFieldCount();
 }
 
@@ -571,6 +585,8 @@ int OGR_F_GetFieldCount( OGRFeatureH hFeat )
 OGRFieldDefnH OGR_F_GetFieldDefnRef( OGRFeatureH hFeat, int i )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_GetFieldDefnRef", NULL );
+
     return (OGRFieldDefnH) ((OGRFeature *) hFeat)->GetFieldDefnRef(i);
 }
 
@@ -612,6 +628,8 @@ OGRFieldDefnH OGR_F_GetFieldDefnRef( OGRFeatureH hFeat, int i )
 int OGR_F_GetFieldIndex( OGRFeatureH hFeat, const char *pszName )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_GetFieldIndex", 0 );
+
     return ((OGRFeature *) hFeat)->GetFieldIndex( pszName );
 }
 
@@ -649,6 +667,8 @@ int OGR_F_GetFieldIndex( OGRFeatureH hFeat, const char *pszName )
 int OGR_F_IsFieldSet( OGRFeatureH hFeat, int iField )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_IsFieldSet", 0 );
+
     return ((OGRFeature *)hFeat)->IsFieldSet( iField );
 }
 
@@ -716,6 +736,8 @@ void OGRFeature::UnsetField( int iField )
 void OGR_F_UnsetField( OGRFeatureH hFeat, int iField )
 
 {
+    VALIDATE_POINTER0( hFeat, "OGR_F_UnsetField" );
+
     ((OGRFeature *) hFeat)->UnsetField( iField );
 }
 
@@ -755,6 +777,8 @@ void OGR_F_UnsetField( OGRFeatureH hFeat, int iField )
 OGRField *OGR_F_GetRawFieldRef( OGRFeatureH hFeat, int iField )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_GetRawFieldRef", NULL );
+
     return ((OGRFeature *)hFeat)->GetRawFieldRef( iField );
 }
 
@@ -838,6 +862,8 @@ int OGRFeature::GetFieldAsInteger( int iField )
 int OGR_F_GetFieldAsInteger( OGRFeatureH hFeat, int iField )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_GetFieldAsInteger", 0 );
+
     return ((OGRFeature *)hFeat)->GetFieldAsInteger(iField);
 }
 
@@ -921,6 +947,8 @@ double OGRFeature::GetFieldAsDouble( int iField )
 double OGR_F_GetFieldAsDouble( OGRFeatureH hFeat, int iField )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_GetFieldAsDouble", 0 );
+
     return ((OGRFeature *)hFeat)->GetFieldAsDouble(iField);
 }
 
@@ -1224,6 +1252,8 @@ const char *OGRFeature::GetFieldAsString( int iField )
 const char *OGR_F_GetFieldAsString( OGRFeatureH hFeat, int iField )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_GetFieldAsString", NULL );
+
     return ((OGRFeature *)hFeat)->GetFieldAsString(iField);
 }
 
@@ -1299,6 +1329,8 @@ const int *OGR_F_GetFieldAsIntegerList( OGRFeatureH hFeat, int iField,
                                   int *pnCount )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_GetFieldAsIntegerList", NULL );
+
     return ((OGRFeature *)hFeat)->GetFieldAsIntegerList(iField, pnCount);
 }
 
@@ -1374,6 +1406,8 @@ const double *OGR_F_GetFieldAsDoubleList( OGRFeatureH hFeat, int iField,
                                           int *pnCount )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_GetFieldAsDoubleList", NULL );
+
     return ((OGRFeature *)hFeat)->GetFieldAsDoubleList(iField, pnCount);
 }
 
@@ -1438,6 +1472,8 @@ char **OGRFeature::GetFieldAsStringList( int iField ) const
 char **OGR_F_GetFieldAsStringList( OGRFeatureH hFeat, int iField )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_GetFieldAsStringList", NULL );
+
     return ((OGRFeature *)hFeat)->GetFieldAsStringList(iField);
 }
 
@@ -1507,6 +1543,8 @@ GByte *OGRFeature::GetFieldAsBinary( int iField, int *pnBytes )
 GByte *OGR_F_GetFieldAsBinary( OGRFeatureH hFeat, int iField, int *pnBytes )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_GetFieldAsBinary", NULL );
+
     return ((OGRFeature *)hFeat)->GetFieldAsBinary(iField,pnBytes);
 }
 
@@ -1607,6 +1645,8 @@ int OGR_F_GetFieldAsDateTime( OGRFeatureH hFeat, int iField,
                               int *pnTZFlag )
     
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_GetFieldAsDateTime", 0 );
+
     return ((OGRFeature *)hFeat)->GetFieldAsDateTime( iField,
                                                       pnYear, pnMonth, pnDay,
                                                       pnHour, pnMinute,pnSecond,
@@ -1686,6 +1726,8 @@ void OGRFeature::SetField( int iField, int nValue )
 void OGR_F_SetFieldInteger( OGRFeatureH hFeat, int iField, int nValue )
 
 {
+    VALIDATE_POINTER0( hFeat, "OGR_F_SetFieldInteger" );
+
     ((OGRFeature *)hFeat)->SetField( iField, nValue );
 }
 
@@ -1762,6 +1804,8 @@ void OGRFeature::SetField( int iField, double dfValue )
 void OGR_F_SetFieldDouble( OGRFeatureH hFeat, int iField, double dfValue )
 
 {
+    VALIDATE_POINTER0( hFeat, "OGR_F_SetFieldDouble" );
+
     ((OGRFeature *)hFeat)->SetField( iField, dfValue );
 }
 
@@ -1841,6 +1885,8 @@ void OGRFeature::SetField( int iField, const char * pszValue )
 void OGR_F_SetFieldString( OGRFeatureH hFeat, int iField, const char *pszValue)
 
 {
+    VALIDATE_POINTER0( hFeat, "OGR_F_SetFieldString" );
+
     ((OGRFeature *)hFeat)->SetField( iField, pszValue );
 }
 
@@ -1901,6 +1947,8 @@ void OGR_F_SetFieldIntegerList( OGRFeatureH hFeat, int iField,
                                 int nCount, int *panValues )
 
 {
+    VALIDATE_POINTER0( hFeat, "OGR_F_SetFieldIntegerList" );
+
     ((OGRFeature *)hFeat)->SetField( iField, nCount, panValues );
 }
 
@@ -1961,6 +2009,8 @@ void OGR_F_SetFieldDoubleList( OGRFeatureH hFeat, int iField,
                                int nCount, double *padfValues )
 
 {
+    VALIDATE_POINTER0( hFeat, "OGR_F_SetFieldDoubleList" );
+
     ((OGRFeature *)hFeat)->SetField( iField, nCount, padfValues );
 }
 
@@ -2019,6 +2069,8 @@ void OGR_F_SetFieldStringList( OGRFeatureH hFeat, int iField,
                                char ** papszValues )
 
 {
+    VALIDATE_POINTER0( hFeat, "OGR_F_SetFieldStringList" );
+
     ((OGRFeature *)hFeat)->SetField( iField, papszValues );
 }
 
@@ -2078,6 +2130,8 @@ void OGR_F_SetFieldBinary( OGRFeatureH hFeat, int iField,
                            int nBytes, GByte *pabyData )
 
 {
+    VALIDATE_POINTER0( hFeat, "OGR_F_SetFieldBinary" );
+
     ((OGRFeature *)hFeat)->SetField( iField, nBytes, pabyData );
 }
 
@@ -2156,6 +2210,8 @@ void OGR_F_SetFieldDateTime( OGRFeatureH hFeat, int iField,
 
 
 {
+    VALIDATE_POINTER0( hFeat, "OGR_F_SetFieldDateTime" );
+
     ((OGRFeature *)hFeat)->SetField( iField, nYear, nMonth, nDay,
                                      nHour, nMinute, nSecond, nTZFlag );
 }
@@ -2324,6 +2380,8 @@ void OGRFeature::SetField( int iField, OGRField * puValue )
 void OGR_F_SetFieldRaw( OGRFeatureH hFeat, int iField, OGRField *psValue )
 
 {
+    VALIDATE_POINTER0( hFeat, "OGR_F_SetFieldRaw" );
+
     ((OGRFeature *)hFeat)->SetField( iField, psValue );
 }
 
@@ -2395,6 +2453,8 @@ void OGRFeature::DumpReadable( FILE * fpOut )
 void OGR_F_DumpReadable( OGRFeatureH hFeat, FILE *fpOut )
 
 {
+    VALIDATE_POINTER0( hFeat, "OGR_F_DumpReadable" );
+
     ((OGRFeature *) hFeat)->DumpReadable( fpOut );
 }
 
@@ -2429,6 +2489,8 @@ void OGR_F_DumpReadable( OGRFeatureH hFeat, FILE *fpOut )
 long OGR_F_GetFID( OGRFeatureH hFeat )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_GetFID", 0 );
+
     return ((OGRFeature *) hFeat)->GetFID();
 }
 
@@ -2482,6 +2544,8 @@ OGRErr OGRFeature::SetFID( long nFID )
 OGRErr OGR_F_SetFID( OGRFeatureH hFeat, long nFID )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_SetFID", CE_Failure );
+
     return ((OGRFeature *) hFeat)->SetFID(nFID);
 }
 
@@ -2546,6 +2610,9 @@ OGRBoolean OGRFeature::Equal( OGRFeature * poFeature )
 int OGR_F_Equal( OGRFeatureH hFeat, OGRFeatureH hOtherFeat )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_Equal", 0 );
+    VALIDATE_POINTER1( hOtherFeat, "OGR_F_Equal", 0 );
+
     return ((OGRFeature *) hFeat)->Equal( (OGRFeature *) hOtherFeat );
 }
 
@@ -2677,6 +2744,9 @@ OGRErr OGR_F_SetFrom( OGRFeatureH hFeat, OGRFeatureH hOtherFeat,
                       int bForgiving )
 
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_SetFrom", CE_Failure );
+    VALIDATE_POINTER1( hOtherFeat, "OGR_F_SetFrom", CE_Failure );
+
     return ((OGRFeature *) hFeat)->SetFrom( (OGRFeature *) hOtherFeat, 
                                            bForgiving );
 }
@@ -2721,6 +2791,8 @@ const char *OGRFeature::GetStyleString()
 
 const char *OGR_F_GetStyleString( OGRFeatureH hFeat )
 {
+    VALIDATE_POINTER1( hFeat, "OGR_F_GetStyleString", NULL );
+
     return ((OGRFeature *)hFeat)->GetStyleString();
 }
 
@@ -2768,6 +2840,8 @@ void OGRFeature::SetStyleString(const char *pszString)
 void OGR_F_SetStyleString( OGRFeatureH hFeat, const char *pszStyle )
 
 {
+    VALIDATE_POINTER0( hFeat, "OGR_F_SetStyleString" );
+
     ((OGRFeature *)hFeat)->SetStyleString( pszStyle );
 }
 
@@ -2811,6 +2885,8 @@ void OGRFeature::SetStyleStringDirectly(char *pszString)
 void OGR_F_SetStyleStringDirectly( OGRFeatureH hFeat, char *pszStyle )
 
 {
+    VALIDATE_POINTER0( hFeat, "OGR_F_SetStyleStringDirectly" );
+
     ((OGRFeature *)hFeat)->SetStyleStringDirectly( pszStyle );
 }
 
@@ -2875,3 +2951,4 @@ OGRErr OGRFeature::RemapFields( OGRFeatureDefn *poNewDefn,
 
     return OGRERR_NONE;
 }
+
