@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrgeometryfactory.cpp 13553 2008-01-20 12:07:33Z rouault $
+ * $Id: ogrgeometryfactory.cpp 15472 2008-10-07 20:48:32Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Factory for converting geometry to and from well known binary
@@ -34,7 +34,7 @@
 #include <assert.h>
 #include "ogr_geos.h"
 
-CPL_CVSID("$Id: ogrgeometryfactory.cpp 13553 2008-01-20 12:07:33Z rouault $");
+CPL_CVSID("$Id: ogrgeometryfactory.cpp 15472 2008-10-07 20:48:32Z rouault $");
 
 /************************************************************************/
 /*                           createFromWkb()                            */
@@ -714,6 +714,62 @@ OGRGeometry *OGRGeometryFactory::forceToMultiLineString( OGRGeometry *poGeom )
 /*                          organizePolygons()                          */
 /************************************************************************/
 
+typedef struct _sPolyExtended sPolyExtended;
+
+struct _sPolyExtended
+{
+    OGRPolygon*     poPolygon;
+    OGREnvelope     sEnvelope;
+    OGRLinearRing*  poExteriorRing;
+    OGRPoint        poAPoint;
+    int             nInitialIndex;
+    int             bIsTopLevel;
+    OGRPolygon*     poEnclosingPolygon;
+    double          dfArea;
+    int             bIsCW;
+};
+
+static int OGRGeometryFactoryCompareArea(const void* p1, const void* p2)
+{
+    const sPolyExtended* psPoly1 = (const sPolyExtended*) p1;
+    const sPolyExtended* psPoly2 = (const sPolyExtended*) p2;
+    if (psPoly2->dfArea < psPoly1->dfArea)
+        return -1;
+    else if (psPoly2->dfArea > psPoly1->dfArea)
+        return 1;
+    else
+        return 0;
+}
+
+static int OGRGeometryFactoryCompareByIndex(const void* p1, const void* p2)
+{
+    const sPolyExtended* psPoly1 = (const sPolyExtended*) p1;
+    const sPolyExtended* psPoly2 = (const sPolyExtended*) p2;
+    if (psPoly1->nInitialIndex < psPoly2->nInitialIndex)
+        return -1;
+    else if (psPoly1->nInitialIndex > psPoly2->nInitialIndex)
+        return 1;
+    else
+        return 0;
+}
+
+#define N_CRITICAL_PART_NUMBER   100
+
+/* Helper function : will be removed in GDAL 1.6.0 */
+OGRGeometry* OGRGeometryFactory::organizePolygons( OGRGeometry **papoPolygons,
+                                                   int nPolygonCount,
+                                                   int *pbIsValidGeometry )
+{
+    return organizePolygons(papoPolygons, nPolygonCount, pbIsValidGeometry, NULL);
+}
+
+typedef enum
+{
+   METHOD_NORMAL,
+   METHOD_SKIP,
+   METHOD_ONLY_CCW
+} OrganizePolygonMethod;
+
 /**
  * Organize polygons based on geometries.
  *
@@ -732,30 +788,37 @@ OGRGeometry *OGRGeometryFactory::forceToMultiLineString( OGRGeometry *poGeom )
  * (unless 'OGR_DEBUG_ORGANIZE_POLYGONS' configuration option is set to TRUE.
  * In that case, a slower algorithm that tests exact topological relationships 
  * is used if GEOS is available.)
- * 
+ *
+ * In cases where a big number of polygons is passed to this function, the default processing
+ * may be really slow. You can skip the processing by adding METHOD=SKIP
+ * to the option list (the result of the function will be a multi-polygon with all polygons
+ * as toplevel polygons) or only make it analyze counterclockwise polygons by adding
+ * METHOD=ONLY_CCW to the option list if you can assume that the outline
+ * of holes is counterclockwise defined (this is the convention for shapefiles e.g.)
+ *
+ * If the OGR_ORGANIZE_POLYGONS configuration option is defined, its value will override
+ * the value of the METHOD option of papszOptions (usefull to modify the behaviour of the
+ * shapefile driver)
+ *
  * @param papoPolygons array of geometry pointers - should all be OGRPolygons.
  * Ownership of the geometries is passed, but not of the array itself.
  * @param nPolygonCount number of items in papoPolygons
  * @param pbIsValidGeometry value will be set TRUE if result is valid or 
  * FALSE otherwise. 
+ * @param papszOptions a list of strings for passing options
  *
  * @return a single resulting geometry (either OGRPolygon or OGRMultiPolygon).
  */
 
-enum
-{
-    CONTAINS,
-    IS_CONTAINED_BY,
-    NOT_RELATED
-};
-
 OGRGeometry* OGRGeometryFactory::organizePolygons( OGRGeometry **papoPolygons,
                                                    int nPolygonCount,
-                                                   int *pbIsValidGeometry )
+                                                   int *pbIsValidGeometry,
+                                                   const char** papszOptions )
 {
     int bUseFastVersion;
     int i, j;
     OGRGeometry* geom = NULL;
+    OrganizePolygonMethod method = METHOD_NORMAL;
 
 /* -------------------------------------------------------------------- */
 /*      Trivial case of a single polygon.                               */
@@ -792,27 +855,54 @@ OGRGeometry* OGRGeometryFactory::organizePolygons( OGRGeometry **papoPolygons,
     }
 
 /* -------------------------------------------------------------------- */
-/*      Setup per polygon relation, envelope and area information.      */
+/*      Setup per polygon envelope and area information.                */
 /* -------------------------------------------------------------------- */
-    OGREnvelope* envelopes = new OGREnvelope[nPolygonCount];
-    int** relations = new int*[nPolygonCount];
-    double* areas = new double[nPolygonCount];
+    sPolyExtended* asPolyEx = new sPolyExtended[nPolygonCount];
+
     int go_on = TRUE;
     int bMixedUpGeometries = FALSE;
     int bNonPolygon = FALSE;
+    int bFoundCCW = FALSE;
+
+    const char* pszMethodValue = CSLFetchNameValue( (char**)papszOptions, "METHOD" );
+    const char* pszMethodValueOption = CPLGetConfigOption("OGR_ORGANIZE_POLYGONS", NULL);
+    if (pszMethodValueOption != NULL && pszMethodValueOption[0] != '\0')
+        pszMethodValue = pszMethodValueOption;
+
+    if (pszMethodValue != NULL)
+    {
+        if (EQUAL(pszMethodValue, "SKIP"))
+        {
+            method = METHOD_SKIP;
+            bMixedUpGeometries = TRUE;
+        }
+        else if (EQUAL(pszMethodValue, "ONLY_CCW"))
+        {
+            method = METHOD_ONLY_CCW;
+        }
+        else if (!EQUAL(pszMethodValue, "DEFAULT"))
+        {
+            CPLError(CE_Warning, CPLE_AppDefined,
+                     "Unrecognized value for METHOD option : %s", pszMethodValue);
+        }
+    }
 
     for(i=0;i<nPolygonCount;i++)
     {
-        relations[i] = new int[nPolygonCount];
-        papoPolygons[i]->getEnvelope(&envelopes[i]);
-
-        //fprintf(stderr, "[%d] %d points\n", i, ((OGRPolygon *)papoPolygons[i])->getExteriorRing()->getNumPoints());
+        asPolyEx[i].nInitialIndex = i;
+        asPolyEx[i].poPolygon = (OGRPolygon*)papoPolygons[i];
+        papoPolygons[i]->getEnvelope(&asPolyEx[i].sEnvelope);
 
         if( wkbFlatten(papoPolygons[i]->getGeometryType()) == wkbPolygon
             && ((OGRPolygon *) papoPolygons[i])->getNumInteriorRings() == 0
             && ((OGRPolygon *)papoPolygons[i])->getExteriorRing()->getNumPoints() >= 4)
         {
-            areas[i] = ((OGRPolygon *)papoPolygons[i])->get_Area();
+            asPolyEx[i].dfArea = asPolyEx[i].poPolygon->get_Area();
+            asPolyEx[i].poExteriorRing = asPolyEx[i].poPolygon->getExteriorRing();
+            asPolyEx[i].poExteriorRing->getPoint(0, &asPolyEx[i].poAPoint);
+            asPolyEx[i].bIsCW = asPolyEx[i].poExteriorRing->isClockwise();
+            if (!bFoundCCW)
+                bFoundCCW = ! (asPolyEx[i].bIsCW);
         }
         else
         {
@@ -828,19 +918,51 @@ OGRGeometry* OGRGeometryFactory::organizePolygons( OGRGeometry **papoPolygons,
             if( wkbFlatten(papoPolygons[i]->getGeometryType()) != wkbPolygon )
                 bNonPolygon = TRUE;
         }
-            
     }
-        
-    /* This a several step algorithm :
-       1) Compute in a matrix how polygons relate to each other
-       (this is the moment for detecting pathological intersections and exiting)
-       2) For each polygon, find the smallest enclosing polygon
-       3) For each polygon, compute its inclusion depth (0 means toplevel)
-       4) For each polygon of odd depth (= inner ring), add it to its outer ring
-        
+
+    /* Emits a warning if the number of parts is sufficiently big to anticipate for */
+    /* very long computation time, and the user didn't specify an explicit method */
+    if (nPolygonCount > N_CRITICAL_PART_NUMBER && method == METHOD_NORMAL && pszMethodValue == NULL)
+    {
+        static int firstTime = 1;
+        if (firstTime)
+        {
+            if (bFoundCCW)
+            {
+                CPLError( CE_Warning, CPLE_AppDefined, 
+                     "organizePolygons() received a polygon with more than %d parts. "
+                     "The processing may be really slow.\n"
+                     "You can skip the processing by setting METHOD=SKIP, "
+                     "or only make it analyze counter-clock wise parts by setting "
+                     "METHOD=ONLY_CCW if you can assume that the "
+                     "outline of holes is counter-clock wise defined", N_CRITICAL_PART_NUMBER);
+            }
+            else
+            {
+                CPLError( CE_Warning, CPLE_AppDefined, 
+                        "organizePolygons() received a polygon with more than %d parts. "
+                        "The processing may be really slow.\n"
+                        "You can skip the processing by setting METHOD=SKIP.",
+                        N_CRITICAL_PART_NUMBER);
+            }
+            firstTime = 0;
+        }
+    }
+
+
+    /* This a several steps algorithm :
+       1) Sort polygons by descending areas
+       2) For each polygon of rank i, find its smallest enclosing polygon
+          among the polygons of rank [i-1 ... 0]. If there are no such polygon,
+          this is a toplevel polygon. Otherwise, depending on if the enclosing
+          polygon is toplevel or not, we can decide if we are toplevel or not
+       3) Re-sort the polygons to retrieve their inital order (nicer for some applications)
+       4) For each non toplevel polygon (= inner ring), add it to its outer ring
+       5) Add the toplevel polygons to the multipolygon
+
        Complexity : O(nPolygonCount^2)
     */
-        
+
     /* Compute how each polygon relate to the other ones
        To save a bit of computation we always begin the computation by a test 
        on the enveloppe. We also take into account the areas to avoid some 
@@ -857,40 +979,111 @@ OGRGeometry* OGRGeometryFactory::organizePolygons( OGRGeometry **papoPolygons,
        no full geometry intersection or inclusion test is done
     */
 
+    if (!bMixedUpGeometries)
+    {
+        /* STEP 1 : Sort polygons by descending area */
+        qsort(asPolyEx, nPolygonCount, sizeof(sPolyExtended), OGRGeometryFactoryCompareArea);
+    }
+    papoPolygons = NULL; /* just to use to avoid it afterwards */
+
 /* -------------------------------------------------------------------- */
 /*      Compute relationships, if things seem well structured.          */
 /* -------------------------------------------------------------------- */
-    for(i=0; !bMixedUpGeometries && go_on && i<nPolygonCount; i++)
-    {
-        for(j=i+1; go_on && j<nPolygonCount;j++)
-        {
-            OGRPoint pointI, pointJ;
-            const OGRLinearRing* exteriorRingI = ((OGRPolygon *)papoPolygons[i])->getExteriorRing();
-            const OGRLinearRing* exteriorRingJ = ((OGRPolygon *)papoPolygons[j])->getExteriorRing();
-            exteriorRingI->getPoint(0, &pointI);
-            exteriorRingJ->getPoint(0, &pointJ);
 
-            if (areas[i] > areas[j] && envelopes[i].Contains(envelopes[j]) &&
-                ((bUseFastVersion && exteriorRingI->isPointInRing(&pointJ)) ||
-                 (!bUseFastVersion && papoPolygons[i]->Contains(papoPolygons[j]))))
+    /* The first (largest) polygon is necessarily top-level */
+    asPolyEx[0].bIsTopLevel = TRUE;
+    asPolyEx[0].poEnclosingPolygon = NULL;
+
+    int nCountTopLevel = 1;
+
+    /* STEP 2 */
+    for(i=1; !bMixedUpGeometries && go_on && i<nPolygonCount; i++)
+    {
+        if (method == METHOD_ONLY_CCW && asPolyEx[i].bIsCW)
+        {
+            nCountTopLevel ++;
+            asPolyEx[i].bIsTopLevel = TRUE;
+            asPolyEx[i].poEnclosingPolygon = NULL;
+            continue;
+        }
+
+        for(j=i-1; go_on && j>=0;j--)
+        {
+            int b_i_inside_j = FALSE;
+
+            if (method == METHOD_ONLY_CCW && asPolyEx[j].bIsCW == FALSE)
             {
-                relations[i][j] = CONTAINS;
-                relations[j][i] = IS_CONTAINED_BY;
+                /* In that mode, i which is CCW if we reach here can only be */
+                /* included in a CW polygon */
+                continue;
             }
-            else if (areas[j] > areas[i] && envelopes[j].Contains(envelopes[i]) &&
-                     ((bUseFastVersion && exteriorRingJ->isPointInRing(&pointI)) ||
-                      (!bUseFastVersion && papoPolygons[j]->Contains(papoPolygons[i]))))
+
+            if (asPolyEx[j].sEnvelope.Contains(asPolyEx[i].sEnvelope))
             {
-                relations[j][i] = CONTAINS;
-                relations[i][j] = IS_CONTAINED_BY;
+                if (bUseFastVersion)
+                {
+                    /* Note that isPointInRing only test strict inclusion in the ring */
+                    if (asPolyEx[j].poExteriorRing->isPointInRing(&asPolyEx[i].poAPoint))
+                    {
+                        b_i_inside_j = TRUE;
+                    }
+                    else if (asPolyEx[j].poExteriorRing->isPointOnRingBoundary(&asPolyEx[i].poAPoint))
+                    {
+                        /* If the point of i is on the boundary of j, we will iterate over the other points of i */
+                        int k, nPoints = asPolyEx[i].poExteriorRing->getNumPoints();
+                        for(k=1;k<nPoints;k++)
+                        {
+                            OGRPoint point;
+                            asPolyEx[i].poExteriorRing->getPoint(k, &point);
+                            if (asPolyEx[j].poExteriorRing->isPointInRing(&point))
+                            {
+                                /* If then point is strictly included in j, then i is considered inside j */
+                                b_i_inside_j = TRUE;
+                                break;
+                            }
+                            else if (asPolyEx[j].poExteriorRing->isPointOnRingBoundary(&point))
+                            {
+                                /* If it is on the boundary of j, iterate again */ 
+                            }
+                            else
+                            {
+                                /* If it is outside, then i cannot be inside j */
+                                break;
+                            }
+                        }
+                    }
+                }
+                else if (asPolyEx[j].poPolygon->Contains(asPolyEx[i].poPolygon))
+                {
+                    b_i_inside_j = TRUE;
+                }
+            }
+
+
+            if (b_i_inside_j)
+            {
+                if (asPolyEx[j].bIsTopLevel)
+                {
+                    /* We are a lake */
+                    asPolyEx[i].bIsTopLevel = FALSE;
+                    asPolyEx[i].poEnclosingPolygon = asPolyEx[j].poPolygon;
+                }
+                else
+                {
+                    /* We are included in a something not toplevel (a lake), */
+                    /* so in OGCSF we are considered as toplevel too */
+                    nCountTopLevel ++;
+                    asPolyEx[i].bIsTopLevel = TRUE;
+                    asPolyEx[i].poEnclosingPolygon = NULL;
+                }
+                break;
             }
             /* We use Overlaps instead of Intersects to be more 
                tolerant about touching polygons */ 
-            else if ( bUseFastVersion || !envelopes[i].Intersects(envelopes[j])
-                     || !papoPolygons[i]->Overlaps(papoPolygons[j]) )
+            else if ( bUseFastVersion || !asPolyEx[i].sEnvelope.Intersects(asPolyEx[j].sEnvelope)
+                     || !asPolyEx[i].poPolygon->Overlaps(asPolyEx[j].poPolygon) )
             {
-                relations[i][j] = NOT_RELATED;
-                relations[j][i] = NOT_RELATED;
+
             }
             else
             {
@@ -902,8 +1095,8 @@ OGRGeometry* OGRGeometryFactory::organizePolygons( OGRGeometry **papoPolygons,
 #ifdef DEBUG
                 char* wkt1;
                 char* wkt2;
-                papoPolygons[i]->exportToWkt(&wkt1);
-                papoPolygons[j]->exportToWkt(&wkt2);
+                asPolyEx[i].poPolygon->exportToWkt(&wkt1);
+                asPolyEx[j].poPolygon->exportToWkt(&wkt2);
                 CPLDebug( "OGR", 
                           "Bad intersection for polygons %d and %d\n"
                           "geom %d: %s\n"
@@ -914,6 +1107,15 @@ OGRGeometry* OGRGeometryFactory::organizePolygons( OGRGeometry **papoPolygons,
 #endif
             }
         }
+
+        if (j < 0)
+        {
+            /* We come here because we are not included in anything */
+            /* We are toplevel */
+            nCountTopLevel ++;
+            asPolyEx[i].bIsTopLevel = TRUE;
+            asPolyEx[i].poEnclosingPolygon = NULL;
+        }
     }
 
     if (pbIsValidGeometry)
@@ -932,7 +1134,7 @@ OGRGeometry* OGRGeometryFactory::organizePolygons( OGRGeometry **papoPolygons,
         for( i=0; i < nPolygonCount; i++ )
         {
             ((OGRGeometryCollection*)geom)->
-                addGeometryDirectly( papoPolygons[i] );
+                addGeometryDirectly( asPolyEx[i].poPolygon );
         }
     }
 
@@ -942,100 +1144,46 @@ OGRGeometry* OGRGeometryFactory::organizePolygons( OGRGeometry **papoPolygons,
 /* -------------------------------------------------------------------- */
     else
     {
-        int* directContainerIndex = new int[nPolygonCount];
+        /* STEP 3: Resort in initial order */
+        qsort(asPolyEx, nPolygonCount, sizeof(sPolyExtended), OGRGeometryFactoryCompareByIndex);
 
-        /* Find the smallest enclosing polygon of each polygon */
+        /* STEP 4: Add holes as rings of their enclosing polygon */
         for(i=0;i<nPolygonCount;i++)
         {
-            int jSmallestContainer = -1;
-            double areaSmallestContainer = 0;
-            for(j=0;j<nPolygonCount;j++)
+            if (asPolyEx[i].bIsTopLevel == FALSE)
             {
-                if (i != j)
+                asPolyEx[i].poEnclosingPolygon->addRing(
+                    asPolyEx[i].poPolygon->getExteriorRing());
+                delete asPolyEx[i].poPolygon;
+            }
+            else if (nCountTopLevel == 1)
+            {
+                geom = asPolyEx[i].poPolygon;
+            }
+        }
+
+        /* STEP 5: Add toplevel polygons */
+        if (nCountTopLevel > 1)
+        {
+            for(i=0;i<nPolygonCount;i++)
+            {
+                if (asPolyEx[i].bIsTopLevel)
                 {
-                    if (relations[i][j] == IS_CONTAINED_BY)
+                    if (geom == NULL)
                     {
-                        if (jSmallestContainer < 0 || areas[j] < areaSmallestContainer)
-                        {
-                            jSmallestContainer = j;
-                            areaSmallestContainer = areas[j];
-                        }
+                        geom = new OGRMultiPolygon();
+                        ((OGRMultiPolygon*)geom)->addGeometryDirectly(asPolyEx[i].poPolygon);
+                    }
+                    else
+                    {
+                        ((OGRMultiPolygon*)geom)->addGeometryDirectly(asPolyEx[i].poPolygon);
                     }
                 }
             }
-            directContainerIndex[i] = jSmallestContainer;
         }
-
-        /* Compute the inclusion depth of each polygon */
-        int* containedDepth = new int [nPolygonCount];
-        for(i=0;i<nPolygonCount;i++)
-        {
-            int depth = 0;
-            int j = directContainerIndex[i];
-            while (j >= 0)
-            {
-                j = directContainerIndex[j];
-                depth++;
-            }
-            containedDepth[i] = depth;
-//          fprintf(stderr, "%d is of depth %d\n", i, depth);
-        }
-
-        int nbTopLevelPolygons = 0;
-        OGRPolygon** tempPolygons = new OGRPolygon*[nPolygonCount]; 
-
-        /* Create a copy of toplevel polygons */
-        for(i=0;i<nPolygonCount;i++)
-        {
-            if ((containedDepth[i] % 2) == 0)
-            {
-                nbTopLevelPolygons ++;
-                tempPolygons[i] = (OGRPolygon*)papoPolygons[i];
-                papoPolygons[i] = NULL;
-                if (nbTopLevelPolygons == 1)
-                    geom = tempPolygons[i];
-            }
-        }
-
-        /* Add interior rings to toplevel polygons */
-        for(i=0;i<nPolygonCount;i++)
-        {
-            if ((containedDepth[i] % 2) == 1)
-            {
-                tempPolygons[directContainerIndex[i]]->addRing(
-                    ((OGRPolygon *)papoPolygons[i])->getExteriorRing());
-                delete papoPolygons[i];
-            }
-        }
-
-        if (nbTopLevelPolygons > 1)
-        {
-            geom = new OGRMultiPolygon();
-
-            /* Add toplevel polygons to the multipolygon */
-            for(i=0;i<nPolygonCount;i++)
-            {
-                if ((containedDepth[i] % 2) == 0)
-                {
-                    ((OGRMultiPolygon*)geom)->addGeometryDirectly(
-                        tempPolygons[i]);
-                    tempPolygons[i] = NULL;
-                }
-            }
-        }
-
-        delete[] tempPolygons;
-        delete[] directContainerIndex;
-        delete[] containedDepth;
     }
 
-    for(i=0;i<nPolygonCount;i++)
-    {
-        delete[] relations[i];
-    }
-    delete[] relations;
-    delete[] areas;
-    delete[] envelopes;
+    delete[] asPolyEx;
 
     return geom;
 }
