@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mitab_tabfile.cpp,v 1.59 2006/02/08 05:02:58 dmorissette Exp $
+ * $Id: mitab_tabfile.cpp,v 1.65 2007/09/12 20:22:31 dmorissette Exp $
  *
  * Name:     mitab_tabfile.cpp
  * Project:  MapInfo TAB Read/Write library
@@ -32,6 +32,28 @@
  **********************************************************************
  *
  * $Log: mitab_tabfile.cpp,v $
+ * Revision 1.65  2007/09/12 20:22:31  dmorissette
+ * Added TABFeature::CreateFromMapInfoType()
+ *
+ * Revision 1.64  2007/06/21 14:00:23  dmorissette
+ * Added missing cast in isspace() calls to avoid failed assertion on Windows
+ * (MITAB bug 1737, GDAL ticket 1678))
+ *
+ * Revision 1.63  2007/06/12 13:52:38  dmorissette
+ * Added IMapInfoFile::SetCharset() method (bug 1734)
+ *
+ * Revision 1.62  2007/06/12 12:50:40  dmorissette
+ * Use Quick Spatial Index by default until bug 1732 is fixed (broken files
+ * produced by current coord block splitting technique).
+ *
+ * Revision 1.61  2007/03/21 21:15:56  dmorissette
+ * Added SetQuickSpatialIndexMode() which generates a non-optimal spatial
+ * index but results in faster write time (bug 1669)
+ *
+ * Revision 1.60  2006/11/28 18:49:08  dmorissette
+ * Completed changes to split TABMAPObjectBlocks properly and produce an
+ * optimal spatial index (bug 1585)
+ *
  * Revision 1.59  2006/02/08 05:02:58  dmorissette
  * Fixed crash when attempting to write TABPolyline object with an invalid
  * geometry (GDAL bug 1059)
@@ -169,7 +191,6 @@ TABFile::TABFile()
     m_pszFname = NULL;
     m_papszTABFile = NULL;
     m_nVersion = 300;
-    m_pszCharset = NULL;
     m_eTableType = TABTableNative;
 
     m_poMAPFile = NULL;
@@ -407,6 +428,7 @@ int TABFile::Open(const char *pszFname, const char *pszAccess,
          * Close() call... we will just set some defaults here.
          *------------------------------------------------------------*/
         m_nVersion = 300;
+        CPLFree(m_pszCharset);
         m_pszCharset = CPLStrdup("Neutral");
         m_eTableType = TABTableNative;
 
@@ -596,6 +618,7 @@ int TABFile::ParseTABFileFirstPass(GBool bTestOpenNoError)
                  * so we set default values for the other params.
                  */
                 bInsideTableDef = TRUE;
+                CPLFree(m_pszCharset);
                 m_pszCharset = CPLStrdup("Neutral");
                 m_eTableType = TABTableNative;
             }
@@ -610,6 +633,7 @@ int TABFile::ParseTABFileFirstPass(GBool bTestOpenNoError)
         }
         else if (EQUAL(papszTok[0], "!charset"))
         {
+            CPLFree(m_pszCharset);
             m_pszCharset = CPLStrdup(papszTok[1]);
         }
         else if (EQUAL(papszTok[0], "Definition") &&
@@ -1106,6 +1130,38 @@ int TABFile::Close()
     return 0;
 }
 
+/**********************************************************************
+ *                   TABFile::SetQuickSpatialIndexMode()
+ *
+ * Select "quick spatial index mode". 
+ *
+ * The default behavior of MITAB is to generate an optimized spatial index,
+ * but this results in slower write speed. 
+ *
+ * Applications that want faster write speed and do not care
+ * about the performance of spatial queries on the resulting file can
+ * use SetQuickSpatialIndexMode() to require the creation of a non-optimal
+ * spatial index (actually emulating the type of spatial index produced
+ * by MITAB before version 1.6.0). In this mode writing files can be 
+ * about 5 times faster, but spatial queries can be up to 30 times slower.
+ *
+ * Returns 0 on success, -1 on error.
+ **********************************************************************/
+int TABFile::SetQuickSpatialIndexMode(GBool bQuickSpatialIndexMode/*=TRUE*/)
+{
+    if (m_eAccessMode != TABWrite || m_poMAPFile == NULL)
+    {
+        CPLError(CE_Failure, CPLE_AssertionFailed,
+                 "SetQuickSpatialIndexMode() failed: file not opened for write access.");
+        return -1;
+    }
+
+
+    return m_poMAPFile->SetQuickSpatialIndexMode(bQuickSpatialIndexMode);
+}
+
+
+
 /**********************************************************************
  *                   TABFile::GetNextFeatureId()
  *
@@ -1265,82 +1321,11 @@ TABFeature *TABFile::GetFeatureRef(int nFeatureId)
 
     /*-----------------------------------------------------------------
      * Create new feature object of the right type
+     * Unsupported object types are returned as raw TABFeature (i.e. NONE
+     * geometry)
      *----------------------------------------------------------------*/
-    switch(m_poMAPFile->GetCurObjType())
-    {
-      case TAB_GEOM_NONE:
-        m_poCurFeature = new TABFeature(m_poDefn);
-        break;
-      case TAB_GEOM_SYMBOL_C:
-      case TAB_GEOM_SYMBOL:
-        m_poCurFeature = new TABPoint(m_poDefn);
-        break;
-      case TAB_GEOM_FONTSYMBOL_C:
-      case TAB_GEOM_FONTSYMBOL:
-        m_poCurFeature = new TABFontPoint(m_poDefn);
-        break;
-      case TAB_GEOM_CUSTOMSYMBOL_C:
-      case TAB_GEOM_CUSTOMSYMBOL:
-        m_poCurFeature = new TABCustomPoint(m_poDefn);
-        break;
-      case TAB_GEOM_LINE_C:
-      case TAB_GEOM_LINE:
-      case TAB_GEOM_PLINE_C:
-      case TAB_GEOM_PLINE:
-      case TAB_GEOM_MULTIPLINE_C:
-      case TAB_GEOM_MULTIPLINE:
-      case TAB_GEOM_V450_MULTIPLINE_C:
-      case TAB_GEOM_V450_MULTIPLINE:
-       m_poCurFeature = new TABPolyline(m_poDefn);
-        break;
-      case TAB_GEOM_ARC_C:
-      case TAB_GEOM_ARC:
-        m_poCurFeature = new TABArc(m_poDefn);
-        break;
-
-      case TAB_GEOM_REGION_C:
-      case TAB_GEOM_REGION:
-      case TAB_GEOM_V450_REGION_C:
-      case TAB_GEOM_V450_REGION:
-        m_poCurFeature = new TABRegion(m_poDefn);
-        break;
-      case TAB_GEOM_RECT_C:
-      case TAB_GEOM_RECT:
-      case TAB_GEOM_ROUNDRECT_C:
-      case TAB_GEOM_ROUNDRECT:
-        m_poCurFeature = new TABRectangle(m_poDefn);
-        break;
-      case TAB_GEOM_ELLIPSE_C:
-      case TAB_GEOM_ELLIPSE:
-        m_poCurFeature = new TABEllipse(m_poDefn);
-        break;
-      case TAB_GEOM_TEXT_C:
-      case TAB_GEOM_TEXT:
-        m_poCurFeature = new TABText(m_poDefn);
-        break;
-      case TAB_GEOM_MULTIPOINT_C:
-      case TAB_GEOM_MULTIPOINT:
-        m_poCurFeature = new TABMultiPoint(m_poDefn);
-        break;
-      case TAB_GEOM_COLLECTION_C:
-      case TAB_GEOM_COLLECTION:
-        m_poCurFeature = new TABCollection(m_poDefn);  
-        break;
-      default:
-        /*-------------------------------------------------------------
-         * Unsupported feature type... we still return a valid feature
-         * with NONE geometry after producing a Warning.
-         * Callers can trap that case by checking CPLGetLastErrorNo() 
-         * against TAB_WarningFeatureTypeNotSupported
-         *------------------------------------------------------------*/
-//        m_poCurFeature = new TABDebugFeature(m_poDefn);
-        m_poCurFeature = new TABFeature(m_poDefn);
-
-        CPLError(CE_Warning, TAB_WarningFeatureTypeNotSupported,
-                 "Unsupported object type %d (0x%2.2x).  Feature will be "
-                 "returned with NONE geometry.", 
-                 m_poMAPFile->GetCurObjType(), m_poMAPFile->GetCurObjType() );
-    }
+    m_poCurFeature = TABFeature::CreateFromMapInfoType(m_poMAPFile->GetCurObjType(), 
+                                                       m_poDefn);
 
     /*-----------------------------------------------------------------
      * Read fields from the .DAT file
@@ -1470,8 +1455,6 @@ int TABFile::SetFeature(TABFeature *poFeature, int nFeatureId /*=-1*/)
     TABMAPObjHdr *poObjHdr = 
         TABMAPObjHdr::NewObj(poFeature->ValidateMapInfoType(m_poMAPFile),
                              nFeatureId);
-    TABMAPObjectBlock *poObjBlock = NULL;
-
     
     /*-----------------------------------------------------------------
      * ValidateMapInfoType() may have returned TAB_GEOM_NONE if feature
@@ -1487,29 +1470,31 @@ int TABFile::SetFeature(TABFeature *poFeature, int nFeatureId /*=-1*/)
         return -1;
     }
 
+    /*-----------------------------------------------------------------
+     * The ValidateMapInfoType() call above has forced calculation of the
+     * feature's IntMBR. Store that value in the ObjHdr for use by
+     * PrepareNewObj() to search the best node to insert the feature.
+     *----------------------------------------------------------------*/
+    if ( poObjHdr && poObjHdr->m_nType != TAB_GEOM_NONE)
+    {
+        poFeature->GetIntMBR(poObjHdr->m_nMinX, poObjHdr->m_nMinY,
+                             poObjHdr->m_nMaxX, poObjHdr->m_nMaxY);
+    }
+
     if ( poObjHdr == NULL || m_poMAPFile == NULL ||
-        m_poMAPFile->PrepareNewObj(nFeatureId, poObjHdr->m_nType) != 0 ||
-         poFeature->WriteGeometryToMAPFile(m_poMAPFile, poObjHdr) != 0 )
+         m_poMAPFile->PrepareNewObj(poObjHdr) != 0 ||
+         poFeature->WriteGeometryToMAPFile(m_poMAPFile, poObjHdr) != 0 ||
+         m_poMAPFile->CommitNewObj(poObjHdr) != 0 )
     {
         CPLError(CE_Failure, CPLE_FileIO,
                  "Failed writing geometry for feature id %d in %s",
                  nFeatureId, m_pszFname);
+        if (poObjHdr)
+            delete poObjHdr;
         return -1;
     }
 
-    if (poObjHdr->m_nType == TAB_GEOM_NONE)
-    {
-        // NONE objects have no reference in the ObjectBlocks.  Just flush it.
-        delete poObjHdr;
-    }
-    else if ( (poObjBlock = m_poMAPFile->GetCurObjBlock()) == NULL ||
-              poObjBlock->AddObject(poObjHdr) != 0 )
-    {
-        CPLError(CE_Failure, CPLE_FileIO,
-                 "Failed writing object header for feature id %d in %s",
-                 nFeatureId, m_pszFname);
-        return -1;
-    }
+    delete poObjHdr;
 
     return nFeatureId;
 }
@@ -2029,7 +2014,6 @@ int TABFile::SetBounds(double dXMin, double dYMin,
     return 0;
 }
 
-
 /**********************************************************************
  *                   TABFile::GetBounds()
  *
