@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrshapedatasource.cpp 11223 2007-04-08 14:32:58Z mloskot $
+ * $Id: ogrshapedatasource.cpp 11928 2007-08-22 20:43:30Z mloskot $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRShapeDataSource class.
@@ -31,7 +31,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrshapedatasource.cpp 11223 2007-04-08 14:32:58Z mloskot $");
+CPL_CVSID("$Id: ogrshapedatasource.cpp 11928 2007-08-22 20:43:30Z mloskot $");
 
 /************************************************************************/
 /*                         OGRShapeDataSource()                         */
@@ -56,7 +56,11 @@ OGRShapeDataSource::~OGRShapeDataSource()
     CPLFree( pszName );
 
     for( int i = 0; i < nLayers; i++ )
+    {
+        CPLAssert( NULL != papoLayers[i] );
+
         delete papoLayers[i];
+    }
     
     CPLFree( papoLayers );
 }
@@ -113,7 +117,7 @@ int OGRShapeDataSource::Open( const char * pszNewName, int bUpdate,
             if( !bTestOpen )
                 CPLError( CE_Failure, CPLE_OpenFailed,
                           "Failed to open shapefile %s.\n"
-                          "It may be corrupt.\n",
+                          "It may be corrupt or read-only file accessed in update mode.\n",
                           pszNewName );
 
             return FALSE;
@@ -147,7 +151,7 @@ int OGRShapeDataSource::Open( const char * pszNewName, int bUpdate,
             {
                 CPLError( CE_Failure, CPLE_OpenFailed,
                           "Failed to open shapefile %s.\n"
-                          "It may be corrupt.\n",
+                          "It may be corrupt or read-only file accessed in update mode.\n",
                           pszFilename );
                 CPLFree( pszFilename );
                 return FALSE;
@@ -208,7 +212,7 @@ int OGRShapeDataSource::Open( const char * pszNewName, int bUpdate,
             {
                 CPLError( CE_Failure, CPLE_OpenFailed,
                           "Failed to open dbf file %s.\n"
-                          "It may be corrupt.\n",
+                          "It may be corrupt or read-only file accessed in update mode.\n",
                           pszFilename );
                 CPLFree( pszFilename );
                 return FALSE;
@@ -571,6 +575,8 @@ OGRShapeDataSource::CreateLayer( const char * pszLayerName,
         }
 
         CPLFree( pszWKT );
+
+        poSRS->morphFromESRI();
     }
 
 /* -------------------------------------------------------------------- */
